<?php

defined('_JEXEC') or die('Restricted access');

/** Import library dependencies */
jimport('joomla.plugin.plugin');

class plgHikaShopAmbrasubs extends JPlugin {

	function plgHikaShopAmbrasubs(& $subject, $config)
	{
		parent::__construct($subject, $config);
	}

	/**
	 * Confirms the component is installed and adds helper files
	 * @return unknown_type
	 */
	function _isInstalled()
	{
		$success = false;
		
		jimport('joomla.filesystem.file');
		if (JFile::exists(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_ambrasubs'.DS.'helpers'.DS.'ambrasubs.php')) 
		{
			// Require Helpers
			jimport( 'joomla.filesystem.folder' );
			$helpersPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_ambrasubs'.DS.'helpers';
			$helperFiles = JFolder::files($helpersPath, '\.php$', false, true);
			if (count($helperFiles) > 0) 
			{
				//iterate through the helper files
				foreach ($helperFiles as $helperFile) {
					require_once($helperFile);
				}
			}
			$success = true;
		}
				
		return $success;
	}
	
	/**
	 * This method is executed after an order status
	 * has been updated in VirtueMart 
	 * 
	 * @param int $orderid
	 * @param string $new_orderstatus
	 * @return boolean|null
	 */
	function onAfterOrderUpdate(&$order,&$send_email){{
		if (!$this->_isInstalled()) {
			return true;
		}

		$this->_executeAdd( $order );
		
		return true;
	}
	
	function onBeforeOrderUpdate(&$order){
		$config=&hikashop::config();
		$confirmed = $config->get('order_confirmed_status');
		if(isset($order->order_status) && $order->order_status==$confirmed){
			$orderClass = hikashop::get('class.order');
			$this->oldOrder = $orderClass->get($order->order_id,false);
		}
	}
	
	/**
	 * Executes Add
	 *
  	 * @param int $orderid
	 * @param string $new_orderstatus
	 * @return boolean|null	 
	 * @access protected
	 */
	function _executeAdd( $order ) 
	{
		$success = false;
		
		$class = hikashop::get('class.user');
		$data = $class->get($order->order_user_id);
		$userid 				= $data->user_cms_id;
		$skus 					= $this->_getSKUs( $order->order_id );
		$cids 					= $this->_getSubscriptionTypes( $skus );
		
		
		$config=&hikashop::config();
		$confirmed = $config->get('order_confirmed_status');
		//if order status is not updated skip
		if(!isset($order->order_status)) return $success;
		
		if($order->order_status!=$confirmed || empty($this->oldOrder)){
			//if new status is not confirmed or that older status was already confirmed just skip
			return $success;
		}
		
		if (empty($cids)) {
			return $success;
		}
		
		// Add the user to the typeids		
			$details = new JObject();
			$details->userid = $userid;
			foreach (@$cids as $cid)
			{
				$details->typeid = $cid;
				
				// type info
				$table_type = AmbrasubsHelperSubscription::getTable( 'Type' );
				$table_type->load( $details->typeid );
				
				// find expiration date 'period' days in future
				$database = JFactory::getDBO();
				$jdate = JFactory::getDate();
				$date = $jdate->toMySQL();
				$query = " SELECT DATE_ADD('{$date}', INTERVAL {$table_type->period} DAY ) ";
				$database->setQuery( $query );
				$expires_datetime = $database->loadResult();
	
				// create the association between user & subscription type
				$row = AmbrasubsHelperSubscription::getTable( 'users2types' );
				$row->userid			= $details->userid;
				$row->typeid			= $details->typeid;
				$row->status			= '1';
				$row->created_datetime	= $jdate->toMySQL();
				$row->expires_datetime	= $expires_datetime;
				
			   	// Store the entry to the database
		    	if (!$row->store()) {
		    		JError::raiseNotice( $row->getError() );
		    		$success = false;
		    	} 
		    	else {
		    		$msg = new stdClass();
		    		$user =& JFactory::getUser($userid);
		    		// fire plugin event for add2juga
		    		JPluginHelper::importPlugin( 'ambrasubs' );
					$dispatcher = JDispatcher::getInstance();
					$dispatcher->trigger( 'onAfterSaveSuccessSubscription', array( $row, $user, $msg ) );
		    	}
			}
			
		return $success;
	}	
	
	/**
	 * Returns the SKUs from an order in an array
	 * 
	 * @param $orderid
	 * @return array
	 */
	function _getSKUs( $orderid ) 
	{
		$result = null;
		
		$db = JFactory::getDBO();
		$query = "
			SELECT 
				order_product_code AS sku
			FROM
				#__hikashop_order_product
			WHERE
				order_id = '{$orderid}'
		";
		$db->setQuery( $query );
		
		if ($data = $db->loadObjectList())
		{
			jimport( 'joomla.utilities.arrayhelper' ); // /joomla/utilities/arrayhelper.php (line 23)
			$result = JArrayHelper::getColumn( $data, 'sku' );
		}
		
		return $result;
	}
		
	/**
	 * Returns an array of subscription type ids that are associated with the specified skus
	 * 
	 * @param $skus array of skus
	 * @return array
	 */
	function _getSubscriptionTypes( $skus ) 
	{
		$result = null;
		
		$sku_query = "tbl.params LIKE '%".implode( "%' OR tbl.params LIKE '%", $skus )."%'";
		
		$db = JFactory::getDBO();
		$query = "
			SELECT 
				tbl.id
			FROM
				#__ambrasubs_types AS tbl
			WHERE
				tbl.params LIKE '%vm_sku=%'
				AND ({$sku_query})
		";
		$db->setQuery( $query );
		if ($data = $db->loadObjectList())
		{
			jimport( 'joomla.utilities.arrayhelper' ); // /joomla/utilities/arrayhelper.php (line 23)
			$result = JArrayHelper::getColumn( $data, 'id' );
		}

		return $result;
	}

}